<?php 

class WPBW_Crawler_Tracking {
	protected $bot = NULL;
	protected $_bots = [
		// Google
		'googlebot',
		'adsbot-google',
		'mediapartners-google',
		'Google Page Speed Insights',

		// Bing
		'bingbot',
		'bingpreview',
		'msnbot',

		// Yahoo!
		'Slurp',

		// Yandex
		'yandex',

		// Facebook
		'Facebot',
		'facebookexternalhit',

		// Twitter
		'Twitterbot',

		// Alexa
		'ia_archiver',

		// Baidu
		'Baiduspider',

		// DuckDuckGo
		'DuckDuckBot',

		// AOL
		'AOLBuild',
	];

	/**
	* Inits the event handlers
	*/
	function __construct() {
		if ($this->shouldAddCannonical()) {
			add_action('wp_head', [$this, 'displayCannonicalUrl']);
		}

		add_action('wp_head', [$this, 'listenForBots']);
	}

	function listenForBots() {
		if ($this->isBot()) {
			// $this->_initBotRef();

			$this->logAction();
		}
	}

	/**
	* Returns the current type
	*/
	protected function _getCurrentUrlType() {
		$queried_object = get_queried_object();

		if ($queried_object instanceof WP_Term) {
			return [
				'type' => $queried_object->taxonomy,
				'id' => $queried_object->term_id,
				'is_page' => 0,
				'name' => apply_filters('the_title', $queried_object->name),
			];
		} else if ($queried_object instanceof WP_Post) {
			return [
				'type' => $queried_object->post_type,
				'id' => $queried_object->ID,
				'is_page' => 1,
				'name' => apply_filters('the_title', $queried_object->post_title),
			];
		} else if (is_object($query_var) && isset($queried_object->name)) {
			return [
				'type' => 'custom',
				'id' => 0,
				'is_page' => 0,
				'name' => $queried_object->label
			];
		} else if (is_search()) {
			return [
				'type' => 'search',
				'id' => 0,
				'is_page' => 0,
				'name' => 'Search',
			];
		} else if (is_archive()) {
			return [
				'type' => 'archive',
				'id' => 0,
				'is_page' => 0,
				'name' => 'Archive',
			];
		} else if (is_paged()) {
			return [
				'type' => 'paged',
				'id' => 0,
				'is_page' => 0,
				'name' => 'Paged',
			];
		}
	}

	protected function logAction() {
		global $wpdb, $table_prefix;

		$data = $this->_getCurrentUrlType();
		$result = $wpdb->get_var("SELECT `url` FROM `". $wpdb->prefix . WPBW_TRACKER_LOG_TABLE ."` WHERE `user_agent` LIKE '%". $this->bot ."%' AND `date` >= DATE_SUB(NOW(), INTERVAL 1 HOUR) ORDER BY date DESC LIMIT 1");

		$wpdb->insert($table_prefix . 'wpbw_tracker_log', [
			'date' => date('Y-m-d H:i:s'),
			'user_agent' => $_SERVER['HTTP_USER_AGENT'],
			'url' => $this->generateCanonicalUrl(),
			'p_type' => $data['type'],
			'p_ID' => $data['id'],
			'title' => $data['name'],
			'is_page' => $data['is_page'],
			// 'referer' => (isset($_GET['WPBW_ref'])) ? $_GET['WPBW_ref'] : '',
			'referer' => (isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : (!empty($result) ? $result : '')),
			'ip' => $_SERVER['REMOTE_ADDR']
		]);
	}

	/* protected function _initBotRef() {
		add_filter('term_link', [$this, 'addBotReferal']);
		add_filter('post_link', [$this, 'addBotReferal']);
		add_filter('post_type_link', [$this, 'addBotReferal']);
		add_filter('nav_menu_link_attributes', [$this, 'addBotReferalToMenu']);
		add_filter('get_pagenum_link', [$this, 'addToPageNum']);
	}
  
	function addToPageNum($url) {
		$currentUrl = $this->generateCanonicalUrl();
		$url = preg_replace('/(\?|\&)WPBW_ref=(.+)/ui', '', $url);

			if (strpos($url, '?') !== FALSE) {
				$url .= '&WPBW_ref=' . urlencode($currentUrl);
			} else {
				$url .= '?WPBW_ref=' . urlencode($currentUrl);
			}
	    
		return $url;
	}

	function addBotReferal($url) {
		$currentUrl = $this->generateCanonicalUrl();

		if (strpos($url, '?') !== FALSE) {
			$url .= '&WPBW_ref=' . urlencode($currentUrl);
		} else {
			$url .= '?WPBW_ref=' . urlencode($currentUrl);
		}

		return $url;
	}

	function addBotReferalToMenu($att) {
		$currentUrl = $this->generateCanonicalUrl();

		if (strpos($att['href'], 'WPBW_ref') !== FALSE) {
			return $att;
		}

		if (strpos($att['href'], '?') !== FALSE) {
			$att['href'] .= '&WPBW_ref=' . urlencode($currentUrl);
		} else {
			$att['href'] .= '?WPBW_ref=' . urlencode($currentUrl);
		}

		return $att;
	} */

	function displayCannonicalUrl() {
	?>
		<!-- Google Bot Tracker -->
		<link rel="canonical" href="<?php echo $this->generateCanonicalUrl();?>" />
		<!-- end of google bot tracker -->
	<?php
	}

	/**
	* Checks if the current user agent is a bot. Returns false in no bot is detected..
	* Applies <b>WPBW_bots_lists</b> to the bots list
	* @return boolena
	*/
	function isBot() {
		$this->_bots = apply_filters('WPBW_bots_lists', $this->_bots);
		$userAgent = $_SERVER['HTTP_USER_AGENT'];

		foreach ($this->_bots as $bot) {
			if (preg_match('/' . $bot . '/i', $userAgent)) {
				$this->bot = $bot;
				return true;
			}
		}

		return false;
	}

	/**
	* Checks if the plugin needs to render a link cannonical tag in the head
	* @todo Make it an option
	* @return boolean
	*/
	function shouldAddCannonical() {
		return true;
	}

	/**
	* Generates the cannonical url, based on the current url
	*/
	function generateCanonicalUrl() {
		$protocol = (strtolower(substr($_SERVER['SERVER_PROTOCOL'], 0, 5)) == 'https') ? 'https://' : 'http://';
		$server = $_SERVER['SERVER_NAME'];

		$url = $protocol . $server . $_SERVER['REQUEST_URI'];
		$urlSegments = parse_url($url);

		$urlClean = $urlSegments['scheme'] . '://' . $urlSegments['host'] . $urlSegments['path'];

		parse_str($urlSegments['query'], $get);

		if (isset($get['WPBW_ref'])) {
			unset($get['WPBW_ref']);
		}

		if (count($get) > 0) {
			$urlClean .= '?' . http_build_query($get);
		}

		return $urlClean;
	}
}
?>