<?php
/**
 * This class generates the google bot flow data
 * 
 * @version 1.0
 * @Author: WPX Hosting
 */
class WPBW_Indexsing_Flow {
	protected $_target = array();
	protected $_passed = array();
	protected $_passedBuild = array();

	/**
	 * Stores the unique nodes
	 * @var type
	 */
	protected $_nodes = array();

	/**
	 * A counter that stores the current index
	 * @var int
	 */
	protected $_currentIndex = 0;

	/**
	 * Stores the colors
	 * @var type
	 */
	protected $_colors = array();

	function fetch($date = '', $date2 = '', $checked = false, $filter = '/') {
		$_colors = array("#a4aaba", "#d7dce2", "#eff0f4", "#d0cdcd", "#eeefea", "#dbdfd9", "#b6b9a3");
		ini_set('memory_limit', '1024M');

		if ($filter != '/') {
			$result = $this->getAllFiltered($date, $date2, $filter);
		} else {
			$result = $this->getAll($date, $date2);
		}

		$this->_buildNodes($result);
		$this->_populateRecursive($result, $checked);

		$formatedNodes = array();


		foreach ($this->_nodes as $path => $node) {
			if ($path == 'googlebot') {
				$formatedNodes[$node] = array(
					'id' => $node,
					'label' => $path,
					'color' => array('background' => "#66FF66"),
				);
			} else {
				$color = array_rand($_colors, 1);

				$formatedNodes[$node] = array(
					'id' => $node,
					'label' => $path,
					'color' => array('background' => $_colors[$color]),
				);
			}
		}

		return $formatedNodes;
	}

	function fetchEdges($date = '', $date2 = '', $checked = false, $filter = '/') {

		ini_set('memory_limit', '1024M');

		if ($filter != '/') {
			$result = $this->getAllFiltered($date, $date2, $filter);
		} else {
			$result = $this->getAll($date, $date2);
		}


		//$result = $this->getFromDB($date);
		$this->_buildNodes($result);
		$this->_populateRecursive($result, $checked);

		return $this->_path;
	}

	protected function _buildNodes($result, $formated = array()) {
		$passed = array();

		foreach ($result as $path => $children) {
			if (count($children) > 0) {
				foreach ($children as $childPath => $childSubPath) {

					if (isset($this->_passedBuild[$childPath]) || isset($this->_passedBuild[$relationsFrom]) || isset($this->_passedBuild[$relationsTo])) {
						continue;
					}

					if (!isset($this->_nodes[$path])) {
						$this->_nodes[$path] = $this->_currentIndex;
						$this->_currentIndex++;
					}

					if (is_array($childSubPath) && count($childSubPath) > 0) {
						$this->_buildNodes($children, $formated);
					}
				}
			}
		}
	}

	protected function _populateRecursive($result, $checked = false, $formated = array()) {
		$passed = array();
		foreach ($result as $path => $children) {
			if (count($children) > 0) {
				foreach ($children as $childPath => $childSubPath) {

					$relationsFrom = md5($path . 'x' . $childPath);
					$relationsTo = md5($childPath . 'x' . $path);

					if ((isset($this->_passedBuild[$childPath]) || isset($this->_passedBuild[$relationsFrom]) || isset($this->_passedBuild[$relationsTo])) && !$checked) {
						continue;
					}

					if ((!isset($this->_nodes[$childPath]) || !isset($this->_nodes[$path])) && !$checked) {
						continue;
					}

					$this->_path[] = array(
						'from' => $this->_nodes[$path],
						'to' => $this->_nodes[$childPath],
						'physics' => 'false',
					);

					$this->_passedBuild[$childPath] = true;
					$this->_passedBuild[$relationsTo] = true;
					$this->_passedBuild[$relationsFrom] = true;


					if (is_array($childSubPath) && count($childSubPath) > 0) {
						$this->_populateRecursive($children, $checked, $formated);
					}
				}
			}
		}
	}

	protected function getAllFiltered($date = '', $date2 = '', $filter = '/') {
		global $wpdb, $table_prefix;

		$url = site_url();
		$url = str_replace(array('https://', 'http://'), '', $url);

		if ($date != '' && $date2 != '') {
			$date = date('Y-m-d', strtotime($date));
			$date2 = date('Y-m-d', strtotime($date2));
			$date = " AND (DATE(`date`) BETWEEN '{$date}' AND '{$date2}')";
		} else if ($date != '' && $date2 == '') {
			$date = date('Y-m-d', strtotime($date));
			$date = " AND DATE(`date`) >= '{$date}'";
		} else if ($date == $date2 && $date != '') {
			$date = date('Y-m-d', strtotime($date));
			$date = " AND DATE(`date`) = '{$date}'";
		}
		if ($filter != '/') {
//            $filter = '/' . $filter . '/';
//            $filter = str_replace('//', '/', $filter);
			$filter = " AND (url LIKE '%{$filter}%' OR referer LIKE '%{$filter}%' ) ";
		}

		$sql = "SELECT * FROM (SELECT * FROM {$table_prefix}wpbw_tracker_log WHERE url LIKE '%{$url}%' {$date} {$filter} ORDER BY referer ASC) as data GROUP BY url, referer";

		$data = $wpdb->get_results($sql);

		$formated = array();

		foreach ($data as $value) {

			if ($value->referer == '') {
				$value->referer = 'googlebot';
			}

			$value->referer = str_replace(array($url, 'https://', 'http://'), '/', $value->referer);
			$value->url = str_replace(array($url, 'https://', 'http://'), '/', $value->url);
			$value->referer = str_replace('///', '/', $value->referer);
			$value->url = str_replace('///', '/', $value->url);

			$formated[$value->referer][] = $value;
		}

		$fullPath = array();

		if (isset($formated)) {
			foreach ($formated as $paths) {

				foreach ($paths as $path) {

					if ($path->url == 'googlebot') {
						continue;
					}

					//echo $path->referer;
					$fullPath[$path->referer][urldecode($path->url)] = $this->_getSubPahts($path->url, $formated);
				}
			}
		}

		return $fullPath;
	}

	protected function getAll($date = '', $date2 = '') {
		global $wpdb, $table_prefix;

		$url = site_url();
		$url = str_replace(array('https://', 'http://'), '', $url);

		if ($date != '' && $date2 != '') {
			$date = date('Y-m-d', strtotime($date));
			$date2 = date('Y-m-d', strtotime($date2));
			$date = " AND (DATE(`date`) BETWEEN '{$date}' AND '{$date2}')";
		} else if ($date != '' && $date2 == '') {
			$date = date('Y-m-d', strtotime($date));
			$date = " AND DATE(`date`) >= '{$date}'";
		} else if ($date == $date2 && $date != '') {
			$date = date('Y-m-d', strtotime($date));
			$date = " AND DATE(`date`) = '{$date}'";
		}
		$sql = "SELECT * FROM (SELECT * FROM {$table_prefix}wpbw_tracker_log WHERE url LIKE '%{$url}%' {$date} ORDER BY referer ASC) as data GROUP BY url, referer";

//        echo $sql;
//        echo ("<pre>");
		$data = $wpdb->get_results($sql);

		$formated = array();
//        if (isset($filter) && $filter != '') {
//            foreach ($data as $value) {
//                if ($value->referer == '') {
//                    $value->referer = 'direct-link';
//                }
//                $value->referer = str_replace(array($url, 'https://', 'http://'), '/', $value->referer);
//                $value->url = str_replace(array($url, 'https://', 'http://'), '/', $value->url);
//                $filter = '/' . $filter . '/';
//                $filter = str_replace('//', '/', $filter);
//                $value->referer = str_replace('///', '/', $value->referer);
//                $value->url = str_replace('///', '/', $value->url);
//                if (($filter == $value->referer) || ($filter == $value->url)) {
//                    $formated[$value->referer][] = $value;
//                }
//            }
//            $fullPath = array();
//            if (isset($formated['/'])) {
//                foreach ($formated['/'] as $path) {
//
//                    if ($path->url == '/') {
//                        continue;
//                    }
//                    $fullPath[urldecode($path->referer)]['/'][urldecode($path->url)] = $this->_getSubPahts($path->url, $formated);
//                }
//            } else {
//                $isEntry++;
//            }
//
//            if (isset($formated)) {
//                foreach ($formated as $path) {
//                    // echo("<pre>");
//                    //var_dump($formated);
//                    // var_dump($path);
//                    // var_dump($formated);
//
//                    if ($path->url == 'direct-link') {
//                        continue;
//                    }
//                    //echo $path->referer;
//                    $fullPath[$path->referer][urldecode($path->url)] = $this->_getSubPahts($path->url, $formated);
//                }
//            } else {
//                $isEntry++;
//            }
//            if ($isEntry == 2) {
//                echo("No entries for given time stamp!");
//            }
//            return $fullPath;
//        } else {
		foreach ($data as $value) {
			if ($value->referer == '') {
				$value->referer = 'googlebot';
			}

			$value->referer = str_replace(array($url, 'https://', 'http://'), '/', $value->referer);
			$value->url = str_replace(array($url, 'https://', 'http://'), '/', $value->url);

			$value->referer = str_replace('///', '/', $value->referer);
			$value->url = str_replace('///', '/', $value->url);
			$formated[$value->referer][] = $value;
		}
		$fullPath = array();
		if (isset($formated['/'])) {
			foreach ($formated['/'] as $path) {

				if ($path->url == '/home/') {
					continue;
				}

				$fullPath['googlebot']['/home/'][urldecode($path->url)] = $this->_getSubPahts($path->url, $formated);
			}
		} else {
			$isEntry++;
		}

		if (isset($formated['googlebot'])) {
			foreach ($formated['googlebot'] as $path) {

				if ($path->url == 'googlebot') {
					continue;
				}

				$fullPath['googlebot'][urldecode($path->url)] = $this->_getSubPahts($path->url, $formated);
			}
		}
		return $fullPath;
	}

	//}

	protected function _getSubPahts($path, $data) {
		$paths = array();
		if (isset($data[$path])) {
			foreach ($data[$path] as $value) {
				if ($path == $value->url) {
					continue;
				}

				$paths[urldecode($value->url)] = $this->_getSubPahts($value->url, $this->$data);
			}
		}
		return $paths;
	}

//
//    protected function getFromDB($date, $date2) {
//
//        global $wpdb, $table_prefix;
//
//        $baseUrl = site_url();
//
//        $formated = array();
//        $ref = array();
//
//        $formated['/'] = $this->getByPath('/', $date, $date2);
//
//        return $formated;
//    }
//
//    protected function getByPath($path, $date = '', $date2 = '') {
//        global $wpdb, $table_prefix;
//
//        if ($this->_counter > 300) {
//            return false;
//        }
//
//        if ($date != '') {
//            $date = ' AND date >= "' . date('Y-m-d', strtotime($date)) . '" AND date <= "' . date('Y-m-d', strtotime($date2)) . '"';
//        }
//
//        if ($path == '/') {
//            $path = site_url() . '/';
//
//            $sql = "SELECT * FROM {$table_prefix}wpbw_tracker_log WHERE referer = '{$path}' OR referer = '' {$date} ORDER BY date ASC";
//        } else {
//            $sql = "SELECT * FROM {$table_prefix}wpbw_tracker_log WHERE referer LIKE '%{$path}' {$date} ORDER BY date ASC";
//        }
//
//        $data = $wpdb->get_results($sql);
//
//        $this->_counter++;
//
//        $array = array();
//        foreach ($data as $key => $value) {
//            $newPath = str_replace(site_url(), '', $value->url);
//            $newPath = preg_replace('/\/[a-z]{2}\//', '', $newPath);
//            $newPath = trim($newPath, '/') . '/';
//
//            if ($newPath == '/' || $newPath == $path) {
//                continue;
//            }
//
//            if (!isset($this->_passed[$newPath])) {
//                $this->_passed[$newPath] = true;
//                $array[$newPath] = $this->getByPath($newPath);
//            }
//        }
//
//        return $array;
//    }
}
