<?php
/**
* This class gets the items for the CSV files
* 
* @version 1.0
* @Author: WPX Hosting
*/
include_once WPBW_TRACKER_PLUGIN_DIR . 'classes/WPBW_Admin_Log.php';

use Classes\WPBW_Admin_Log as WPBW_Admin_Log;

class CSV_Exporter {
    
    /**
     * gets the items for the csv files
     * @param array $request
     * @return array $data
     */
    function get_table_items($request) {
        $list_name = $request['file_name'];
        $data = array();
        switch ($list_name) {
            case WPBW_CSV_ADMIN_HISTORY:
                $data = $this->get_admin_history_items($request);
                break;
            case WPBW_CSV_EVENT_CHRONOLOGY:
                $data = $this->get_event_chronology_items($request);
                break;
            case WPBW_CSV_ADMIN_ACTIONS:
                $data = $this->get_admin_actions_items($request);
                break;
            case WPBW_CSV_BOT_TRAJECTORY:
                $data = $this->get_bot_trajectory_items($request);
                break;
            case WPBW_CSV_CRAWL_COUNTER:
                $data = $this->get_crawl_counter_items($request);
                break;
        }
        
        return $data;
        
    }
    /**
     * gets the items for the admin_history.csv
     * @param array $request
     * @return array $data
     */
    function get_admin_history_items($request) {

        global $wpdb, $table_prefix;
        $table_actions = $table_prefix . WPBW_TRACKER_ACTIONS_TABLE;
        $table_actions_contexts = $table_prefix . WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE;

        $search = ( isset($request['s']) ) ? $request['s'] : '';
        $start_date = ( isset($request['start_date']) ) ? $request['start_date'] : '';
        $end_date = ( isset($request['end_date']) ) ? $request['end_date'] : '';
        if ((isset($request['start_date'])) && ( $request['end_date'] == '')) {
            $end_date = date('Y-m-d');
        }
        if ((isset($request['end_date'])) && ( $request['start_date'] == '')) {
            $start_date = date('Y-m-d', strtotime('10.10.2010'));
        }
        $orderby = ( $request['orderby_'] != '' ) ? $request['orderby_'] : 'date';
        $order = ( $request['order_'] != '' ) ? $request['order_'] : 'DESC';
        $orderStr = '';

        if ($orderby != '' && $order != '') {
            $orderStr = 'ORDER BY ' . $orderby . ' ' . $order;
        }
        $select = 'SELECT h.id, c.value, h.message, h.date';

        $where = array(
            '1 = 1'
        );

        $joins = array();
        if (($start_date != '') && ($end_date != '')) {

            $start_date = date('Y-m-d', strtotime($start_date));
            $end_date = date('Y-m-d', strtotime($end_date));

            $where['date'] = " (DATE(`date`) BETWEEN '" . $start_date . "'  AND '" . $end_date . "')";
        }

        if ($search != '') {
            $fields = array("c.value", "h.message");

            foreach ($fields as &$item) {
                $item = $item . " LIKE '%" . esc_sql(like_escape($search)) . "%'";
            }

            $fields[] = 'contex_search.value LIKE "%' . $wpdb->esc_like($search) . '%"';

            $where['search'] = ' (' . implode(" OR ", $fields) . ')';
        }
        $where['_user_login'] = 'c.key LIKE "_user_login"';      
        $joins[$table_actions_contexts] = 'INNER JOIN ' . $table_actions_contexts . ' AS c ON h.id = c.history_id' ;
        if ($search != '') {
            $joins['search_for'] = 'INNER JOIN ' . $table_actions_contexts . ' AS contex_search ON h.id = contex_search.history_id';
        }
        $sql = $select . ' FROM ' . $table_actions . ' as h ';
        $sql .= ' ' . implode(' ', $joins) . ' WHERE ' . implode(' AND ', $where) . ' GROUP BY h.id ' . $orderStr;
        $rows = $wpdb->get_results($sql);
        
        $data = array();
        foreach ($rows as $row => $log) {
            $action = $log->message;
            $history_id = $log->id;
            $new_message = (strpos($action, '{')) ? WPBW_Admin_Log::substituteKeysWithValues($history_id, $action) : $action;
            $data[$row]['id'] = $history_id;
            $data[$row]['value'] = $log->value;
            $data[$row]['message'] = $new_message;
            $data[$row]['date'] = $log->date;

            $data[$row] = (object) $data[$row];
        }
        return $data;
    }
    /**
     * gets the items for the event_chronology.csv
     * @param array $request
     * @return array $data
     */
    function get_event_chronology_items($request) {
        global $wpdb, $table_prefix;
        $table = $table_prefix . WPBW_TRACKER_LOG_TABLE;
        
        $search = ( isset($request['s']) ) ? $request['s'] : '';
        $start_date = ( isset($request['start_date']) ) ? $request['start_date'] : '';
        $end_date = ( isset($request['end_date']) ) ? $request['end_date'] : '';
        if ((isset($request['start_date'])) && ( $request['end_date'] == '')) {
            $end_date = date('Y-m-d');
        } 
        if ((isset($request['end_date'])) && ( $request['start_date'] == '')) {
            $start_date = date('Y-m-d', strtotime('10.10.2010'));
        }
        $p_types = ( isset($request['p_type']) ) ? $request['p_type'] : '';
        $user_agent = ( isset($request['user_agent']) ) ? $request['user_agent'] : '';
        $orderby = ( $request['orderby_'] != '' ) ? $request['orderby_'] : 'date';
        $order = ( $request['order_'] != '' ) ? $request['order_'] : 'DESC';
        $orderStr = '';

        if ($orderby != '' && $order != '') {
            $orderStr = ' ORDER BY ' . $orderby . ' ' . $order;
        }

        
        $where = ' WHERE 1=1';
        
        if (($start_date != '') && ($end_date != '')) {

            $start_date = date('Y-m-d', strtotime($start_date));
            $end_date = date('Y-m-d', strtotime($end_date));

            $where .= " AND (DATE(`date`) BETWEEN '" . $start_date . "'  AND '" . $end_date . "')";
        }
        
        
        
        if (isset($p_types) && is_array($p_types) ) {
            foreach ($p_types as &$type) {
                $type = "p_type LIKE '" . $type . "'";
            }
            $where .= ' AND (' . implode(" OR ", $p_types).')';

        }
        
        if ($user_agent !='') {
            $where .= " AND user_agent LIKE '" . $user_agent . "'"; 
        }
        
        if ($search != '') {
            $fields = array("p_ID", "title", "url", "referer");
            foreach ($fields as &$item) {
                $item = $item . " LIKE '%" . esc_sql(like_escape($search)) . "%'";
            }

            $where .= ' AND (' . implode(" OR ", $fields).')';
        }
        
        $sql_str = 'SELECT ID, date, user_agent, url, p_ID, p_type, title, referer FROM '. $table . $where . $orderStr;

        $data = $wpdb->get_results($sql_str);

        return $data;
    }
    /**
     * gets the items for the admin_actions.csv
     * @param array $request
     * @return array $data
     */
    
    function get_admin_actions_items($request) {
        global $wpdb, $table_prefix;
        $table_actions = $table_prefix . WPBW_TRACKER_ACTIONS_TABLE;
        $table_actions_contexts = $table_prefix . WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE;
        $table_gbot = $table_prefix . WPBW_TRACKER_LOG_TABLE;
        $sql = 'SELECT date, p_ID, p_type, is_page FROM '. $table_gbot . ' WHERE id = '. $request['ID'];
        $result = $wpdb->get_row($sql);
        
        $date = $result->date;
        
        $search = ( isset($request['s']) ) ? $request['s'] : '';
        
        
        $orderby = ( isset($request['orderby_']) ) ? $request['orderby_'] : 'date';
        $order = ( isset($request['order_']) ) ? $request['order_'] : 'DESC';
        $orderStr = '';

        if ($orderby != '' && $order != '') {
            $orderStr = 'ORDER BY ' . $orderby . ' ' . $order;
        }

        $select = 'SELECT h.id, c.value, h.message, h.date';

        $where = array(
            '1 = 1'
        );
        
        $where['date'] = " h.date < '" . $date . "'";
        
        $where['_user_login'] = 'c.key LIKE "_user_login"';
        $joins = array();
        if ($result->is_page == 1) {
            $joins['post_id'] = 'INNER JOIN ' . $table_actions_contexts . ' AS contex_post ON h.id = contex_post.history_id';
         
            $where['post_id_key'] = 'contex_post.key LIKE "post_id"';
            $where['post_id_value'] = 'contex_post.value = "' .$result->p_ID. '"';
        }
        
        $joins[$table_actions_contexts] = 'INNER JOIN ' . $table_actions_contexts . ' AS c ON h.id = c.history_id' ;
        
        $sql = $select . ' FROM ' . $table_actions . ' as h ';

        $sql .= ' ' . implode(' ', $joins) . ' WHERE ' . implode(' AND ', $where) . ' GROUP BY h.id ' . $orderStr ;

        $rows = $wpdb->get_results($sql);
       
        $data = array();
        foreach ($rows as $row => $log) {
            $action = $log->message;
            $history_id = $log->id;
            $new_message = (strpos($action, '{')) ? WPBW_Admin_Log::substituteKeysWithValues($history_id, $action) : $action;
            $data[$row]['id'] = $history_id;
            $data[$row]['value'] = $log->value;
            $data[$row]['message'] = $new_message;
            $data[$row]['date'] = $log->date;

            $data[$row] = (object) $data[$row];
        }

        return $data;
}
    /**
     * gets the items for the bot_trajectory.csv
     * @param array $request
     * @return array $data
     */
    function get_bot_trajectory_items($request) {

        global $wpdb, $table_prefix;
        $table = $table_prefix . WPBW_TRACKER_LOG_TABLE;
         
        $orderby = ( $request['orderby_'] != '' ) ? $request['orderby_'] : 'date';
        $order = ( $request['order_'] != '' ) ? $request['order_'] : 'DESC';
       
        $orderStr = '';

        if ($orderby != '' && $order != '') {
            $orderStr = " ORDER BY `" . $orderby . "` " . $order;
        }

        $where = '';
        
        $sql = "SELECT user_agent, date FROM ". $table. " WHERE ID = ".$request['ID'];

        $row = $wpdb->get_row( $sql );
        $date= substr($row->date, 0, 10);
        $sql = "SELECT * FROM ".$table. " WHERE user_agent LIKE '".$row->user_agent."' AND date BETWEEN '".$date." 00:00:00' AND '".$date." 23:59:59'" . $orderStr;
       
        $data = $wpdb->get_results($sql);

        return $data;
        
    }
    /**
     * gets the items for the crawl_counter.csv
     * @param array $request
     * @return array $data
     */
    function get_crawl_counter_items($request) {

        global $wpdb, $table_prefix;
        $table = $table_prefix . WPBW_TRACKER_LOG_TABLE;

        $search = ( isset($request['s']) ) ? $request['s'] : '';
        $start_date = ( isset($request['start_date']) ) ? $request['start_date'] : '';
        $end_date = ( isset($request['end_date']) ) ? $request['end_date'] : '';
        if ((isset($request['start_date'])) && ( $request['end_date'] == '')) {
            $end_date = date('Y-m-d');
        }
        if ((isset($request['end_date'])) && ( $request['start_date'] == '')) {
            $start_date = date('Y-m-d', strtotime('10.10.2010'));
        }
        $orderby = ( $request['orderby_'] != '' ) ? $request['orderby_'] : 'from';
        $order = ( $request['order_'] != '' ) ? $request['order_'] : 'DESC';
       
        $orderStr = '';

        if ($orderby != '' && $order != '') {
            $orderStr = " ORDER BY `" . $orderby . "` " . $order;
        }

        $where = ' WHERE 1=1';
        
        if (($start_date != '') && ($end_date != '')) {
            $start_date = date('Y-m-d', strtotime($start_date));
            $end_date = date('Y-m-d', strtotime($end_date));
            $where .= " AND (DATE(`date`) BETWEEN '" . $start_date . "'  AND '" . $end_date . "')";
        }
        
        if ($search != '') {
            $fields = array("p_ID", "p_type", "url", "title", "user_agent");
            foreach ($fields as &$item) {
                $item = $item . " LIKE '%" . esc_sql(like_escape($search)) . "%'";
            }

            $where .= ' AND (' . implode(" OR ", $fields) . ')';
        }
        
        $sql_str = "SELECT `p_ID`, `title`, `user_agent`, MIN(`date`) as 'from', MAX(`date`) as 'to', COUNT(DISTINCT `ID`) AS crawls FROM ";
        $sql_str .= $table. $where . " GROUP BY `p_ID`, `user_agent` " ;
        $sql_str .= $orderStr;

        $data = $wpdb->get_results($sql_str);
        
        return $data;    
    }
    
}
?>
