<?php
/**
 * This class extends WP_List_Table.
 * Author: WPX Hosting
 * Author URI: https://wpxhosting.com/
 */

include_once WPBW_TRACKER_PLUGIN_DIR . 'classes/WPBW_Admin_Log.php';

use Classes\WPBW_Admin_Log as WPBW_Admin_Log;

if (!class_exists('WP_List_Table')) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Admin_Actions_list extends WP_List_Table {
	function __construct() {
		global $status, $page;

		//Set parent defaults
		parent::__construct(array(
			'singular'	=> 'action', //singular name of the listed records
			'plural'	=> 'actions', //plural name of the listed records
			'ajax'		=> false //does this table support ajax?
		));
	}

	function column_default($item, $column_name) {
		return $item->$column_name;
	}

	function get_columns() {
		$columns = array(
			// 'cb'		=> '<input type="checkbox" />',
			'date'		=> __('Date', 'gbt'),
			'id'		=> __('ID', 'gbt'),
			'value'		=> __('Admin Name', 'gbt'),
			'message'	=> __('Action', 'gbt'),
		);

		return $columns;
	}

	function column_cb($item) {
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/* $1%s */ $this->_args['singular'], //Let's simply repurpose the table's singular label ("reservation")
			/* $2%s */ $item->ID //The value of the checkbox should be the record's id
		);
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'date'		=> array('date', false),
			'id'		=> array('id', false),
			'value'		=> array('value', false),
			'message'	=> array('message', false),
		);

		return $sortable_columns;
	}

	/**
	 * 
	 * @global wpdb $wpdb
	 * @global string $table_prefix
	 */
	function prepare_items() {
		global $wpdb, $table_prefix;
		$table_actions = $table_prefix . WPBW_TRACKER_ACTIONS_TABLE;
		$table_actions_contexts = $table_prefix . WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE;
		$table_gbot = $table_prefix . WPBW_TRACKER_LOG_TABLE;
		$per_page = 30;
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array($columns, $hidden, $sortable);
		$this->process_bulk_action();
		$sql = 'SELECT date, p_ID, p_type, is_page FROM '. $table_gbot . ' WHERE id = '. $_GET['ID'];
		$result = $wpdb->get_row($sql);
		// $date = substr($result->date, 0, 10);
		$date = $result->date;

		$search = ( isset($_REQUEST['s']) ) ? $_REQUEST['s'] : '';
		
		
		$orderby = ( isset($_REQUEST['orderby']) ) ? $_REQUEST['orderby'] : 'date';
		$order = ( isset($_REQUEST['order']) ) ? $_REQUEST['order'] : 'DESC';
		//var_dump($_REQUEST);
		$orderStr = '';

		if ($orderby != '' && $order != '') {
			$orderStr = 'ORDER BY ' . $orderby . ' ' . $order;
		}

		$page = $this->get_pagenum();

		$offset = ($page - 1) * $per_page;

		$select = 'SELECT SQL_CALC_FOUND_ROWS h.id, c.value, h.message, h.date';

		$where = array(
			'1 = 1'
		);
		
		$where['date'] = " h.date < '" . $date . "'";
		
		if ($search != '') {
			$fields = array("c.value", "h.message");
			
			foreach ($fields as &$item) {
				$item = $item . " LIKE '%" . esc_sql(like_escape($search)) . "%'";
			}
			
			$fields[] = 'contex_search.value LIKE "%' . $wpdb->esc_like($search) . '%"';
			
			$where['search'] = ' (' . implode(" OR ", $fields) . ')';
		}
		
		$where['_user_login'] = 'c.key LIKE "_user_login"';
		$joins = array();
		if ($result->is_page == 1) {
			$joins['post_id'] = 'INNER JOIN ' . $table_actions_contexts . ' AS contex_post ON h.id = contex_post.history_id';
//          
			$where['post_id_key'] = 'contex_post.key LIKE "post_id"';
			$where['post_id_value'] = 'contex_post.value = "' .$result->p_ID. '"';
		}
		
		$joins[$table_actions_contexts] = 'INNER JOIN ' . $table_actions_contexts . ' AS c ON h.id = c.history_id' ;
		if ($search != '') {
			$joins['search_for'] = 'INNER JOIN ' . $table_actions_contexts . ' AS contex_search ON h.id = contex_search.history_id';
		}

		$sql = $select . ' FROM ' . $table_actions . ' as h ';

		$sql .= ' ' . implode(' ', $joins) . ' WHERE ' . implode(' AND ', $where) . ' GROUP BY h.id ' . $orderStr . ' LIMIT ' . $offset . ',' . $per_page;
//        echo 'SQL = '.$sql;
		$rows = $wpdb->get_results($sql);
		
		$sql = "SELECT FOUND_ROWS() as total";
		$result = $wpdb->get_row( $sql );
		$total_items = $result->total;
		
		$data = array();
		foreach ($rows as $row => $log) {
			$action = $log->message;
			$history_id = $log->id;
			$new_message = (strpos($action, '{')) ? WPBW_Admin_Log::substituteKeysWithValues($history_id, $action) : $action;
			$data[$row]['date'] = $log->date;
			$data[$row]['id'] = $history_id;
			$data[$row]['value'] = $log->value;
			$data[$row]['message'] = $new_message;

			$data[$row] = (object) $data[$row];
		}

		$this->items = $data;
		
		$this->set_pagination_args(array(
			'total_items' => $total_items, //WE have to calculate the total number of items
			'per_page' => $per_page, //WE have to determine how many items to show on a page
			'total_pages' => ceil($total_items / $per_page)   //WE have to calculate the total number of pages
		));
	}

}