<?php
/**
 * This class extends WP_List_Table.
 * Author: WPX Hosting
 * Author URI: https://wpxhosting.com/
 */
include_once WPBW_TRACKER_PLUGIN_DIR .'classes/WPBW_Admin_Log.php';

use Classes\WPBW_Admin_Log as WPBW_Admin_Log;

if (!class_exists('WP_List_Table')) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Admin_History_list extends WP_List_Table {
	function __construct() {
		global $status, $page;

		//Set parent defaults
		parent::__construct(array(
			'singular' => 'action', //singular name of the listed records
			'plural' => 'actions', //plural name of the listed records
			'ajax' => false        //does this table support ajax?
		));
	}

	function column_default($item, $column_name) {
		return $item->$column_name;
	}

	function get_columns() {
		$columns = array(
			// 'cb' => '<input type="checkbox" />',
			'date' => __('Date', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The exact date and time that you or the Webmaster of this blog performed an action here so that you can compare this with Googlebot activity (or a lack of) on the Events Chronology page."></span>',
			'value' => __('Admin Name', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The username associated with the actions or events on this blog though not relevant if you don\'t have multiple  users with admin privileges here."></span>',
			'message' => __('Action', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The exact action carried out by an admin on this blog e.g. adding posts, updating plugins, logging in etc."></span>',
			'id' => __('Incident ID', 'gbt') .' <span class="wp-botwatch-info tooltip" title="This ID # represents a specific action or event on your blog such as the webmaster - you probably - changing/adding/deleting something on this site or Googlebot crawling your blog."></span>',
		);

		return $columns;
	}

	function column_cb($item) {
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/* $1%s */ $this->_args['singular'], //Let's simply repurpose the table's singular label ("reservation")
			/* $2%s */ $item->ID //The value of the checkbox should be the record's id
		);
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'date'		=> array('date', false),
			'value'		=> array('value', false),
			'message'	=> array('message', false),
			'id'		=> array('id', false),
		);

		return $sortable_columns;
	}

	/**
	 * 
	 * @global wpdb $wpdb
	 * @global string $table_prefix
	 */
	function prepare_items() {
		global $wpdb, $table_prefix;
		$table_actions = $table_prefix . WPBW_TRACKER_ACTIONS_TABLE;
		$table_actions_contexts = $table_prefix . WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE;

		$per_page = 30;

		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array($columns, $hidden, $sortable);

		$this->process_bulk_action();

		$search = ( isset($_REQUEST['s']) ) ? $_REQUEST['s'] : '';
		$start_date = ( isset($_REQUEST['start_date']) ) ? $_REQUEST['start_date'] : '';
		$end_date = ( isset($_REQUEST['end_date']) ) ? $_REQUEST['end_date'] : '';
		if ((isset($_REQUEST['start_date'])) && ( $_REQUEST['end_date'] == '')) {
			$end_date = date('Y-m-d');
		}
		if ((isset($_REQUEST['end_date'])) && ( $_REQUEST['start_date'] == '')) {
			$start_date = date('Y-m-d', strtotime('10.10.2010'));
		}
		$orderby = ( isset($_REQUEST['orderby']) ) ? $_REQUEST['orderby'] : 'h.date';
		$order = ( isset($_REQUEST['order']) ) ? $_REQUEST['order'] : 'DESC';
		//var_dump($_REQUEST);
		$orderStr = '';

		if ($orderby != '' && $order != '') {
			$orderStr = 'ORDER BY '. $orderby .' '. $order;
		}

		$page = $this->get_pagenum();
		$offset = ($page - 1) * $per_page;
		$select = 'SELECT SQL_CALC_FOUND_ROWS h.id, c.value, h.message, h.date';
		$where = array(
			'1 = 1'
		);

		$joins = array();

		if (($start_date != '') && ($end_date != '')) {
			$start_date = date('Y-m-d', strtotime($start_date));
			$end_date = date('Y-m-d', strtotime($end_date));
			$where['date'] = " (DATE(`date`) BETWEEN '". $start_date ."'  AND '". $end_date ."')";
		}

		if ($search != '') {
			$fields = array("c.value", "h.message");
			foreach ($fields as &$item) {
				$item = $item ." LIKE '%". esc_sql(like_escape($search)) ."%'";
			}
			$fields[] = 'contex_search.value LIKE "%' . $wpdb->esc_like($search) .'%"';
			$where['search'] = ' ('. implode(" OR ", $fields) .')';
		}

		$where['_user_login'] = 'c.key LIKE "_user_login"';      
		$joins[$table_actions_contexts] = 'INNER JOIN '. $table_actions_contexts .' AS c ON h.id = c.history_id' ;
		if ($search != '') {
			$joins['search_for'] = 'INNER JOIN '. $table_actions_contexts .' AS contex_search ON h.id = contex_search.history_id';
		}

		$sql = $select .' FROM '. $table_actions .' as h ';

		$sql .= ' '. implode(' ', $joins) .' WHERE '. implode(' AND ', $where) .' GROUP BY h.id '. $orderStr .' LIMIT '. $offset .','. $per_page;
		// echo $sql;
		// echo 'SQL = '.$sql .'<br />';

		$rows = $wpdb->get_results($sql);

		$sql = "SELECT FOUND_ROWS() as total";
		$result = $wpdb->get_row($sql);
		$total_items = $result->total;

		$data = array();
		foreach ($rows as $row => $log) {
			$action = $log->message;
			$history_id = $log->id;
			$new_message = (strpos($action, '{')) ? WPBW_Admin_Log::substituteKeysWithValues($history_id, $action) : $action;
			$data[$row]['date'] = $log->date;
			$data[$row]['value'] = $log->value;
			$data[$row]['message'] = $new_message;
			$data[$row]['id'] = $history_id;
			$data[$row] = (object) $data[$row];
		}

		$this->items = $data;

		// $sql = 'SELECT COUNT(*) as Total FROM '. $table_actions .' AS h  INNER JOIN '. $table_actions_contexts .' AS c ON h.id = c.history_id  WHERE '. implode(' AND ', $where);
		// echo 'SQL = '. $sql;
		// $wpdb->query($sql);
		// $total_items = $wpdb->last_result[0]->Total;

		$this->set_pagination_args(array(
			'total_items'	=> $total_items, //WE have to calculate the total number of items
			'per_page'		=> $per_page, //WE have to determine how many items to show on a page
			'total_pages'	=> ceil($total_items / $per_page) //WE have to calculate the total number of pages
		));
	}
}