<?php
/**
 * This class extends WP_List_Table.
 * Author: WPX Hosting
 * Author URI: https://wpxhosting.com/
 */

if (!class_exists('WP_List_Table')) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Crawl_Counter_list extends WP_List_Table {
	function __construct() {
		global $status, $page;

		//Set parent defaults
		parent::__construct(array(
			'singular'	=> 'crawl', //singular name of the listed records
			'plural'	=> 'crawls', //plural name of the listed records
			'ajax'		=> false //does this table support ajax?
		));
	}

	function column_default($item, $column_name) {
		return $item->$column_name;
	}

	function get_columns() {
		$columns = array(
			// 'cb'			=> '<input type="checkbox" />',
			'from'			=> __('From', 'gbt') .' <span class="wp-botwatch-info tooltip" title="This is the exact date and time that Googlebot arrived on your site for this particular crawl."></span>',
			'to'			=> __('To', 'gbt') .' <span class="wp-botwatch-info tooltip" title="This is the exact date and time that Googlebot left your site on this particular crawl (speedy little devil isn\'t it?)."></span>',
			'p_ID'			=> __('Post ID', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The WordPress ID # of the Post or Page that Googlebot visited first on this crawl."></span>',
			'title'			=> __('Title', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The title of the Page or Post where Googlebot first arrived on this specific crawl."></span>',
			'user_agent'	=> __('User Agent', 'gbt') .' <span class="wp-botwatch-info tooltip" title="Googlebot comes in different forms and here you can see the variation that visited your blog during this crawl."></span>',
			'ip'			=> __('IP', 'gbt') .' <span class="wp-botwatch-info tooltip" title="Here you can see the Googlebot IP address."></span>',
			'crawls'		=> __('Crawls', 'gbt') .' <span class="wp-botwatch-info tooltip" title="This is the number of times that Googlebot has crawled this specific Post/Page on your blog in the timeframe selected on this page - this quickly shows you which areas of your site Googlebot likes and visits often and which ones Googlebot is struggling to crawl - interesting right?"></span>',
		);

		return $columns;
	}

	function column_cb($item) {
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/* $1%s */ $this->_args['singular'], //Let's simply repurpose the table's singular label ("reservation")
			/* $2%s */ $item->ID //The value of the checkbox should be the record's id
		);
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'from'			=> array('from', false), 
			'to'			=> array('to', false),
			'p_ID'			=> array('p_ID', false),
			'title'			=> array('title', false),
			'user_agent'	=> array('user_agent', false),
			'ip'			=> array('ip', false),
			'crawls'		=> array('crawls', false),
		);

		return $sortable_columns;
	}

	function prepare_items() {
		global $wpdb, $table_prefix;

		$table = $table_prefix . WPBW_TRACKER_LOG_TABLE;
		$per_page = 30;
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array($columns, $hidden, $sortable);
		$this->process_bulk_action();

		$search = (isset($_REQUEST['s'])) ? $_REQUEST['s'] : '';
		$start_date = (isset($_REQUEST['start_date'])) ? $_REQUEST['start_date'] : '';
		$end_date = (isset($_REQUEST['end_date'])) ? $_REQUEST['end_date'] : '';

		if ((isset($_REQUEST['start_date'])) && ($_REQUEST['end_date'] == '')) {
			$end_date = date('Y-m-d');
		}

		if ((isset($_REQUEST['end_date'])) && ($_REQUEST['start_date'] == '')) {
			$start_date = date('Y-m-d', strtotime('10.10.2010'));
		}

		$orderby = (isset($_REQUEST['orderby'])) ? $_REQUEST['orderby'] : 'from';
		$order = (isset($_REQUEST['order'])) ? $_REQUEST['order'] : 'DESC';
		$orderStr = '';

		if ($orderby != '' && $order != '') {
			$orderStr = " ORDER BY `". $orderby ."` ". $order;
		}

		$page = $this->get_pagenum();
		$offset = ($page - 1) * $per_page;
		$where = ' WHERE 1=1';

		if (($start_date != '') && ($end_date != '')) {
			$start_date = date('Y-m-d', strtotime($start_date));
			$end_date = date('Y-m-d', strtotime($end_date));
			$where .= " AND (DATE(`date`) BETWEEN '". $start_date ."'  AND '". $end_date ."')";
		}

		if ($search != '') {
			$fields = array("p_ID", "p_type", "url", "title", "user_agent", "ip");

			foreach ($fields as &$item) {
				$item = $item ." LIKE '%". esc_sql(like_escape($search)) ."%'";
			}

			$where .= ' AND ('. implode(" OR ", $fields) .')';
		}

		$sql_str = "SELECT SQL_CALC_FOUND_ROWS `p_ID`, `title`, `user_agent`, `ip`, MIN(`date`) as 'from', MAX(`date`) as 'to', COUNT(DISTINCT `ID`) AS crawls FROM ";
		$sql_str .= $table . $where ." GROUP BY `p_ID`, `user_agent` " ;
		$sql_str .= $orderStr .' LIMIT '. $offset .','. $per_page;
		$data = $wpdb->get_results($sql_str);
		$this->items = $data;

		$total = $wpdb->get_row('SELECT FOUND_ROWS() as total');
		$total_items = $total->total;

		$this->set_pagination_args(array(
			'total_items'	=> $total_items, //WE have to calculate the total number of items
			'per_page'		=> $per_page, //WE have to determine how many items to show on a page
			'total_pages'	=> ceil($total_items / $per_page) //WE have to calculate the total number of pages
		));
	}
}