<?php
/**
 * This class extends WP_List_Table.
 * Author: WPX Hosting
 * Author URI: https://wpxhosting.com/
 */

if (!class_exists('WP_List_Table')) {
	require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class WPBW_Event_Chronology_list extends WP_List_Table {
	function __construct() {
		global $status, $page;

		//Set parent defaults
		parent::__construct(array(
			'singular'	=> 'event', //singular name of the listed records
			'plural'	=> 'events', //plural name of the listed records
			'ajax'		=> false //does this table support ajax?
		));
	}

	function column_default($item, $column_name) {
		return $item->$column_name;
	}

	function get_columns() {
		$columns = array(
			// 'cb'			=> '<input type="checkbox" />',
			'date'			=> __('Date', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The specific date and time that Googlebot visited this blog."></span>',
			'user_agent'	=> __('User Agent', 'gbt') .' <span class="wp-botwatch-info tooltip" title="Googlebot comes in different forms and here you can see the variation that visited your blog during this crawl."></span>',
			'url'			=> __('URL', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The original page on your blog that Googlebot started its crawl on this particular visit to your site."></span>',
			'p_ID'			=> __('Post ID', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The WordPress ID # of the Post or Page that Googlebot visited first on this crawl."></span>',
			'p_type'		=> __('Post or Page', 'gbt') .' <span class="wp-botwatch-info tooltip" title="Shows you whether it was a WP Post or Page that Googlebot began crawling on this visit."></span>',
			'title'			=> __('Title', 'gbt') .' <span class="wp-botwatch-info tooltip" title="The title of the Page or Post where Googlebot first arrived on this specific crawl."></span>',
			'referer'		=> __('Referring URL', 'gbt') .' <span class="wp-botwatch-info tooltip" title="Here you can see the URL that Googlebot came from to the first page URL where the crawl started (see that column in this table) though in some cases this information isn\'t technically possible to acquire and you will see a blank cell in that case."></span>',
			'ip'			=> __('IP', 'gbt') .' <span class="wp-botwatch-info tooltip" title="Here you can see the Googlebot IP address."></span>',
			'ID'			=> __('Incident ID', 'gbt') .' <span class="wp-botwatch-info tooltip" title="This ID # represents a specific action or event on your blog such as the webmaster - you probably - changing/adding/deleting something on this site or Googlebot crawling your blog."></span>'
		);
		return $columns;
	}

	function column_cb($item) {
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/* $1%s */ $this->_args['singular'], //Let's simply repurpose the table's singular label ("reservation")
			/* $2%s */ $item->ID //The value of the checkbox should be the record's id
		);
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'date'			=> array('date', false),
			'user_agent'	=> array('user_agent', false),
			'url'			=> array('url', false), 
			'p_ID'			=> array('p_ID', false),
			'p_type'		=> array('p_type', false),
			'title'			=> array('title', false),
			'referer'		=> array('referer', false),
			'ip'			=> array('ip', false),
			'ID'			=> array('ID', false)
		);
		return $sortable_columns;
	}

	function prepare_items() {
		global $wpdb, $table_prefix;
		$table = $table_prefix . WPBW_TRACKER_LOG_TABLE;

		$per_page = 30;

		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array($columns, $hidden, $sortable);

		$this->process_bulk_action();

		$search = (isset($_REQUEST['s'])) ? $_REQUEST['s'] : '';
		$start_date = (isset($_REQUEST['start_date'])) ? $_REQUEST['start_date'] : '';
		$end_date = (isset($_REQUEST['end_date'])) ? $_REQUEST['end_date'] : '';

		if ((isset($_REQUEST['start_date'])) && ($_REQUEST['end_date'] == '')) {
			$end_date = date('Y-m-d');
		}

		if ((isset($_REQUEST['end_date'])) && ($_REQUEST['start_date'] == '')) {
			$start_date = date('Y-m-d', strtotime('10.10.2010'));
		}

		$p_types = (isset($_REQUEST['p_type'])) ? $_REQUEST['p_type'] : '';
		$user_agent = (isset($_REQUEST['user_agent'])) ? $_REQUEST['user_agent'] : '';
		$orderby = (isset($_REQUEST['orderby'])) ? $_REQUEST['orderby'] : 'date';
		$order = (isset($_REQUEST['order'])) ? $_REQUEST['order'] : 'DESC';
		$orderStr = '';

		if ($orderby != '' && $order != '') {
			$orderStr = ' ORDER BY '. $orderby .' '. $order;
		}

		$page = $this->get_pagenum();
		$offset = ($page - 1) * $per_page;
		$where = ' WHERE 1=1';

		if (($start_date != '') && ($end_date != '')) {
			$start_date = date('Y-m-d', strtotime($start_date));
			$end_date = date('Y-m-d', strtotime($end_date));

			$where .= " AND (DATE(`date`) BETWEEN '". $start_date ."'  AND '". $end_date ."')";
		}

		if (isset($p_types) && is_array($p_types)) {
			foreach ($p_types as &$type) {
				$type = "p_type LIKE '". $type ."'";
			}
			$where .= ' AND ('. implode(" OR ", $p_types) .')';
		}

		if ($user_agent != '') {
			$where .= " AND user_agent LIKE '". $user_agent ."'"; 
		}
		
		if ($search != '') {
			$fields = array("p_ID", "title", "url", "referer");
			foreach ($fields as &$item) {
				$item = $item ." LIKE '%". esc_sql(like_escape($search)) ."%'";
			}

			$where .= ' AND ('. implode(" OR ", $fields) .')';
		}

		$sql_str = 'SELECT SQL_CALC_FOUND_ROWS * FROM '. $table . $where . $orderStr .' LIMIT '. $offset .','. $per_page;

		$data = $wpdb->get_results($sql_str);

		$this->items = $data;
		$sql = "SELECT FOUND_ROWS() as total";
		$result = $wpdb->get_row($sql);
		$total_items = $result->total;
		// $wpdb->query('SELECT COUNT(*) as Total FROM '. $table . $where);
		// $total_items = $wpdb->last_result[0]->Total;

		$this->set_pagination_args(array(
			'total_items'	=> $total_items, //WE have to calculate the total number of items
			'per_page'		=> $per_page, //WE have to determine how many items to show on a page
			'total_pages'	=> ceil($total_items / $per_page) //WE have to calculate the total number of pages
		));
	}

	function column_ID($item) {
		$actions = array(
			'admin_actions'	=> sprintf('<a href="?page=%s&action=%s&ID=%s">Admin Actions</a>', $_REQUEST['page'], 'admin_actions', $item->ID),
			'follow'		=> sprintf('<a href="?page=%s&action=%s&ID=%s">Follow GBot</a>', $_REQUEST['page'], 'follow', $item->ID),
		);

		return sprintf('%1$s %2$s', $item->ID, $this->row_actions($actions));
	}
}