<?php
/*
Plugin Name: WP BotWatch
Description: From Terry Kyle and the WPX Hosting team, WP BotWatch provides detailed information on exactly where, how and how often Googlebot crawls your WP blog (later versions will monitor other bot/crawler activity but this free 1.0 version only monitors Googlebot).
Version: 1.0.4
Author: WPX Hosting
Author URI: https://wpxhosting.com/
*/

define('WPBW_VERSION', '1.0.4');
define('WPBW_TRACKER_PLUGIN_DIR', dirname(__FILE__) .'/');
define('WPBW_TRACKER_LOG_TABLE', 'wpbw_tracker_log');
define('WPBW_TRACKER_ACTIONS_TABLE', 'wpbw_tracker_admin_history');
define('WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE', 'wpbw_tracker_admin_history_contexts');

define('WPBW_CSV_ADMIN_HISTORY', 'admin_history');
define('WPBW_CSV_EVENT_CHRONOLOGY', 'event_chronology');
define('WPBW_CSV_ADMIN_ACTIONS', 'admin_actions');
define('WPBW_CSV_BOT_TRAJECTORY', 'bot_trajectory');
define('WPBW_CSV_CRAWL_COUNTER', 'crawl_counter');

if (!defined('WPBW_ADMIN_HISTORY_PATH')) {
	define('WPBW_ADMIN_HISTORY_PATH', plugin_dir_path(__FILE__));
}

if (!defined('WPBW_ADMIN_HISTORY_BASENAME')) {
	define('WPBW_ADMIN_HISTORY_BASENAME', plugin_basename(__FILE__));
}

require_once('plugin-update/plugin-update-checker.php');
include_once WPBW_TRACKER_PLUGIN_DIR .'classes/Google_Bot_Tracker.php';
include_once WPBW_TRACKER_PLUGIN_DIR .'classes/WPBW_Admin_Log.php';
require_once WPBW_TRACKER_PLUGIN_DIR .'classes/WPBW_Crawler_Tracking.php';

use Classes\Google_Bot_Tracker as GBT;
use Classes\WPBW_Admin_Log as WPBW_Admin;

$crawlerDetection = new WPBW_Crawler_Tracking();

if (!function_exists('wpbw_tracker_register_plugin_hook')) {
	register_activation_hook(__FILE__, 'wpbw_tracker_register_plugin_hook');

	/**
	 * Inits the plugin and creates the needed table
	 * @global wpdb $wpdb
	 * @global string $table_prefix
	 */
	function wpbw_tracker_register_plugin_hook() {
		global $wpdb, $table_prefix;
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		$table_googlebot = $table_prefix . WPBW_TRACKER_LOG_TABLE;
		$table_admin_log = $table_prefix . WPBW_TRACKER_ACTIONS_TABLE;
		$table_contexts = $table_prefix . WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE;


		$sql = "CREATE TABLE IF NOT EXISTS `{$table_googlebot}` (
			`ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
			`date` datetime NOT NULL,
			`user_agent` varchar(500) NOT NULL,
			`url` varchar(500) NOT NULL,
			`p_ID` int(10) DEFAULT NULL,
			`p_type` varchar(250) NOT NULL,
			`title` varchar(255) NOT NULL,
			`referer` varchar(500) NOT NULL,
			`ip` VARCHAR(255) NOT NULL,
			`is_page` tinyint(1) NOT NULL DEFAULT '0',
			PRIMARY KEY (`ID`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1";
		dbDelta($sql);

		$sql = "CREATE TABLE IF NOT EXISTS {$table_admin_log} (
			id bigint(20) NOT NULL AUTO_INCREMENT,
			date datetime NOT NULL,
			logger varchar(30) DEFAULT NULL,
			level varchar(20) DEFAULT NULL,
			message varchar(255) DEFAULT NULL,
			occasionsID varchar(32) DEFAULT NULL,
			initiator varchar(16) DEFAULT NULL,
			PRIMARY KEY  (id),
			KEY date (date),
			KEY loggerdate (logger,date)
		) CHARSET=utf8;";
		dbDelta($sql);

		// Add context table
		$sql = "CREATE TABLE IF NOT EXISTS {$table_contexts} (
			context_id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
			history_id bigint(20) unsigned NOT NULL,
			`key` varchar(255) DEFAULT NULL,
			value longtext,
			PRIMARY KEY  (context_id),
			KEY history_id (history_id),
			KEY `key` (`key`)
		) CHARSET=utf8;";
		dbDelta($sql);

		add_option('wpbw_version', WPBW_VERSION);
	}
}


if (!function_exists('wpbw_tracker_update')) {
	add_action('plugins_loaded', 'wpbw_tracker_update');

	function wpbw_tracker_update() {
		global $wpdb, $table_prefix;
		require_once(ABSPATH .'wp-admin/includes/upgrade.php');

		$table_googlebot = $table_prefix . WPBW_TRACKER_LOG_TABLE;

		if (version_compare(get_option('wpbw_version'), '1.0.3') < 0) {
			$sql = "ALTER TABLE `{$table_googlebot}` ADD `ip` VARCHAR(255) NOT NULL AFTER `is_page`";
			$wpdb->query($sql);
			update_option('wpbw_version', '1.0.3');
		}
	}
}

/**
 * Actions on plugin uninstall
 * @global $wpdb, $table_prefix
 */
if (!function_exists('pluginUninstall')) {
	register_uninstall_hook(__FILE__, 'pluginUninstall');

	function pluginUninstall() {
		global $wpdb, $table_prefix;
		$table_googlebot = $table_prefix . WPBW_TRACKER_LOG_TABLE;
		$table_admin_log = $table_prefix . WPBW_TRACKER_ACTIONS_TABLE;
		$table_contexts = $table_prefix . WPBW_TRACKER_ACTIONS_CONTEXTS_TABLE;

		$wpdb->query("DROP TABLE IF EXISTS $table_admin_log");
		$wpdb->query("DROP TABLE IF EXISTS $table_contexts");
		$wpdb->query("DROP TABLE IF EXISTS $table_googlebot");
	}
}

if (!function_exists('load_WPBW_admin_styles')) {
	add_action('admin_init', 'load_WPBW_admin_styles');

	/**
	 * Enqueue slyles for the admin page
	 */
	function load_WPBW_admin_styles() {
		wp_register_style('wp-botwatch', plugins_url('wp-botwatch/assets/css/style.css'));
		wp_enqueue_style('wp-botwatch');
	}
}

/**
 * if user is admin boots up a class (get from Simple History Plugin) for registering admin actions
 */
// if (is_admin()) {
	require_once(__DIR__ . "/classes/WPBW_Admin_History.php");

	/** Boot up */
	WPBW_Admin_History::get_instance();
// }

if (!function_exists('wpbw_tracker_admin_menu')) {
	add_action('admin_menu', 'wpbw_tracker_admin_menu');

	/**
	 * Creates the admin menu
	 */
	function wpbw_tracker_admin_menu() {
		add_menu_page('WP BotWatch', 'WP BotWatch', 'administrator', 'wpbw_tracker_settings', 'wpbw_tracker_settings', plugins_url('/assets/images/wp-botwatch-icon.png', __FILE__));
		add_submenu_page('wpbw_tracker_settings', 'How It Works', 'How It Works', 'administrator', 'wpbw_how_it_works', 'wpbw_how_it_works');
		add_submenu_page('wpbw_tracker_settings', 'Admin Actions History', 'Admin Actions History', 'administrator', 'wpbw_tracker_admin_action_table', 'wpbw_tracker_admin_action_table_render');
		add_submenu_page('wpbw_tracker_settings', 'Events Chronology', 'Events Chronology', 'administrator', 'wpbw_tracker_events_table', 'wpbw_tracker_events_table_render');
		add_submenu_page('wpbw_tracker_settings', 'Crawl Counter', 'Crawl Counter', 'administrator', 'wpbw_tracker_crawls_counter_table', 'wpbw_tracker_crawls_counter_table_render');
		add_submenu_page('wpbw_tracker_settings', 'Googlebot Crawling Flows', 'Googlebot Crawling Flows', 'administrator', 'wpbw_indexsing_flow', 'wpbw_indexsing_flow');
	}
}

if (!function_exists('adjust_WPBW_menu')) {
	add_action('admin_menu', 'adjust_WPBW_menu', 999);

	/**
	 * Adjust the plugin menu
	 * @global type $menu
	 * @global type $submenu
	 */
	function adjust_WPBW_menu() {
//        $page = remove_submenu_page('wpbw_tracker_settings', 'wpbw_trajectory_page');
		global $menu;
		global $submenu;

		$submenu['wpbw_tracker_settings'][0][0] = 'Clear Logs';
		$submenu['wpbw_tracker_settings'][6] = $submenu['wpbw_tracker_settings'][0];
		$temp_array = array();
		foreach ($submenu['wpbw_tracker_settings'] as $key  => $val) {
			if ($key > 0) {
				$temp_array[$key - 1] = $val;
			}
		}
		$submenu['wpbw_tracker_settings'] = $temp_array;
	}
}

if (!function_exists('wpbw_tracker_settings')) {
	/**
	 * render the settings page
	 */
	function wpbw_tracker_settings() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'templates/settings_page.php';
	}
}

if (!function_exists('wpbw_csv_export_action')) {
	add_action('admin_init', 'wpbw_csv_export_action');

	function wpbw_csv_export_action() {
		if (isset($_GET['export-csv']) && isset($_GET['file_name'])) {
			include_once WPBW_TRACKER_PLUGIN_DIR .'classes/csv_exporter.php';
			$csv_exporter = new CSV_Exporter();
			$list_items = $csv_exporter->get_table_items($_REQUEST);
			$list_name = $_GET['file_name'];

			switch ($list_name) {
				case "admin_history":
					$table_columns = array("ID", "Admin_Name", "Action", "Date");
					WPBW_export_csv($list_items, $list_name, $table_columns);
					break;
				case "event_chronology":
					$table_columns = array("ID", "Date", "User_Agent", "URL", "Post_ID", "Type", "Name", "Referer", "IP");
					WPBW_export_csv($list_items, $list_name, $table_columns);
					break;
				case "admin_actions":
					$table_columns = array("ID", "Admin_Name", "Action", "Date");
					WPBW_export_csv($list_items, $list_name, $table_columns);
					break;
				case "bot_trajectory":
					$table_columns = array("ID", "Date", "User_Agent", "URL", "Post_ID", "Type", "Name", "Referer", "IP");
					WPBW_export_csv($list_items, $list_name, $table_columns);
					break;
				case "crawl_counter":
					$table_columns = array("Post_ID", "Title", "User_Agent", "From", "To", "Crawls");
					WPBW_export_csv($list_items, $list_name, $table_columns);
					break;
			}
		}
	}
}

/**
 * Write to .csv file
 * @param type $list
 * @param type $table_name
 * @param type $table_columns
 */
function WPBW_export_csv($list, $table_name, $table_columns) {
	$file_name = $table_name .'.csv';

	// output headers so that the file is downloaded rather than displayed
	header('Content-Type: text/csv; charset=utf-8');
	header('Content-Disposition: attachment; filename='.$file_name);
	header("Pragma: no-cache");
	header("Expires: 0");

	// create a file pointer connected to the output stream
	$output = fopen('php://output', 'w');

	// output the column headings
	fputcsv($output, $table_columns);
	foreach ($list as $item) {
		$array_list = (array) $item;

		fputcsv($output, $array_list);
	}

	fclose($output);
	die();
}

if (!function_exists('wpbw_how_it_works')) {
	/**
	 * render the GoogleBot Indexing Flow page
	 */
	function wpbw_how_it_works() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'templates/wpbw_how_it_works.php';
	}
}

if (!function_exists('wpbw_tracker_admin_action_table_render')) {
	/**
	 * render the Admin History page
	 */
	function wpbw_tracker_admin_action_table_render() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'classes/table_lists/admin_history_list.php';
		$table = new Admin_History_list();
		$table->prepare_items();

		include_once WPBW_TRACKER_PLUGIN_DIR .'templates/admin_history.php';
	}
}

if (!function_exists('wpbw_tracker_events_table_render')) {
	/**
	 * render the Event Chronology page
	 */
	function wpbw_tracker_events_table_render() {
		if (isset($_GET['action']) && $_GET['action'] == 'follow') {
			WPBW_follow_trajectory();
		} else if (isset($_GET['action']) && $_GET['action'] == 'admin_actions') {
			WPBW_admin_actions();
		} else {
			require_once WPBW_TRACKER_PLUGIN_DIR .'classes/table_lists/event_chronology_list.php';
			$table = new WPBW_Event_Chronology_list();
			$table->prepare_items();
			include_once WPBW_TRACKER_PLUGIN_DIR .'templates/event_chronology.php';
		}
	}
}

if (!function_exists('wpbw_tracker_crawls_counter_table_render')) {
	/**
	 * render the Crawls Counter page
	 */
	function wpbw_tracker_crawls_counter_table_render() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'classes/table_lists/crawl_counter_list.php';
		$table = new Crawl_Counter_list();
		$table->prepare_items();
		include_once WPBW_TRACKER_PLUGIN_DIR .'templates/crawl_counter.php';
	}
}

if (!function_exists('WPBW_follow_trajectory')) {
	/**
	 * render the Follow Trajectory page
	 */
	function WPBW_follow_trajectory() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'classes/table_lists/bot_trajectory_list.php';
		$table = new Bot_Trajectory_list();
		$table->prepare_items();
		include_once WPBW_TRACKER_PLUGIN_DIR .'templates/bot_trajectory.php';
	}
}

if (!function_exists('WPBW_admin_actions')) {
	/**
	 * render the Admin Actions page
	 */
	function WPBW_admin_actions() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'classes/table_lists/admin_actions_list.php';
		$table = new Admin_Actions_list();
		$table->prepare_items();
		include_once WPBW_TRACKER_PLUGIN_DIR .'templates/admin_actions.php';
	}
}

if (!function_exists('wpbw_indexsing_flow')) {
	/**
	 * render the GoogleBot Indexing Flow page
	 */
	function wpbw_indexsing_flow() {
		require_once WPBW_TRACKER_PLUGIN_DIR .'classes/WPBW_Indexsing_Flow.php';
		require_once WPBW_TRACKER_PLUGIN_DIR .'templates/wpbw_indexsing_flow.php';
	}
}

if (!function_exists('WPBW_init_localization')) {
	add_action('plugins_loaded', 'WPBW_init_localization');

	/**
	 * Loads the text domain
	 */
	function WPBW_init_localization() {
		load_plugin_textdomain('gbt', false, plugin_basename(dirname(__FILE__)) .'/languages');
	}
}

if (!function_exists('WPBW_clear_logs')) {
	/**
	 * Clears the log table and display messages
	 */
	function WPBW_clear_logs() {
		if (($_POST['log'] != '') && ($_POST['log'] == 'Admin')) {
			$data = WPBW_Admin::deleteAdminActionsLog();
			if (($data['result_actions'] === false) || ($data['result_context'] === false)) {
				echo __('Problem apeared when trying to delete GoogleBot log.', 'gbt');
			} else if (((int) $data['result_actions'] >= 0) && ($data['action'] == 'delete')) {
				echo (int) $data['result_actions'];
				if ((int) $data['result_actions'] == 1) {
					echo __(' row from Admin Log was deleted.', 'gbt');
				} else {
					echo __(' rows from Admin Log were deleted.', 'gbt');
				}
			} else if (($data['result_actions'] === true) && ($data['result_context'] === true) && ($data['action'] == 'truncate')) {
				echo __('Admin Log succesfully deleted.', 'gbt');
			}
		} else if (($_POST['log'] != '') && ($_POST['log'] == 'GoogleBot')) {
			$data = GBT::deleteLogData();
			$result = $data['result'];
			$action = $data['action'];
			if ($data['error'] != '') {
				echo $data['error'];
			}
			if ($result === false) {
				echo __('Problem apeared when trying to delete GoogleBot log.', 'gbt');
			} else if (((int) $result >= 0) && ($action == 'delete')) {
				echo (int) $result;
				if ((int) $result == 1) {
					echo __(' row from GoogleBot Log was deleted.', 'gbt');
				} else {
					echo __(' rows from GoogleBot Log were deleted.', 'gbt');
				}
			} else if (($result === true) && ($action == 'truncate')) {
				echo __('GoogleBot Log succesfully deleted.', 'gbt');
			}
		}
	}
}

if (!function_exists('init_WPBW_autoloader')) {
	/**
	 * Autoloads the plugin classes
	 * @param type $class
	 */
	function init_WPBW_autoloader($class) {
		$class = str_replace('\\', '/', $class);
		$class = explode('/', $class);
		$className = array_pop($class);
		$path = WPBW_TRACKER_PLUGIN_DIR . strtolower(implode('/', $class)) .'/'. strtolower($className) .'.php';

		if (file_exists($path)) {
			require_once $path;
		}
	}
	spl_autoload_register('init_WPBW_autoloader');
}

$myUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
	'https://plugins.terrykyle.com/?action=get_metadata&slug=wp-botwatch',
	__FILE__,
	'wp-botwatch'
);
